C     RDPG MODEL
C     MDLCLC.FOR
C     CALCULATE THE TARGET FUNCTION


      SUBROUTINE MDLCLC(X, TARGET)

C     DECLARE VARIABLES
C     X: 4 RAIN-DAY PARAMETERS
      REAL X(1:4)
C     RAIN_MODEL: MODEL RAINFALL DATA      
      REAL RAIN_MODEL(1:100)         
C     RAIN_REALDATA: REAL RAINFALL DATA      
      REAL RAIN_REALDATA(1:100)
C     TARGET: TARGET FUNCTION 
      REAL TARGET             
C     VOLUME_MODEL,VOLUME_REALDATA: VOLUME OF MODEL & REAL DATA 
      REAL VOLUME_MODEL, VOLUME_REALDATA
C     T: TIME, 15 MINUTE SPACED ON DAILY BASIS
      REAL T
C     V_DIFFERENCE: VOLUME DIFFERENCE B/W MODEL AND REAL DATA
      REAL*8 V_DIFFERENCE
C     TERM1, TERM2: TWO PARTS IN CONVOLUTION EQUATION
      REAL*8 TERM1, TERM2
C     Y1, Y2, Y3, Y4, Y5, Y6: MIDDLE RESULTS
      REAL*8 Y1, Y2, Y3, Y4, Y5, Y6
C     Y1, Y2, Y3, Y4, Y5, Y6: MIDDLE RESULTS
      REAL*8 Z1, Z2, Z3, Z4, Z5, Z6

C     COMMONLY USED VARAIBLES IN FILES
      COMMON /COM_1/  RAIN_REALDATA
      COMMON /COM_2/  RAIN_MODEL

C     X(1): TIME TO PEAK, X(2): RAIN DISPERSION COEFFICIENT
C     X(3): RAIN DURATION, X(4): PEAK RATE

C     INITIALIZE VARIABLES 
      TARGET = 0.0000
      VOLUME_MODEL = 0.000
      VOLUME_REALDATA = 0.000

C     CONTROL RANGE OF PARAMETERS 
      IF ( X(2) .LE. 0.01) THEN
       X(2) = 0.01
      END IF
      IF ( X(2) .GE. 1.0) THEN
       X(2) = 1.0
      END IF
       
      IF ( X(1) .LE. 15.0) THEN
       X(1) = 15.0
      END IF
      IF ( X(1) .GE. 1440.0) THEN
       X(1) = 1440.0
      END IF
       
      IF ( X(3) .LE. 15.0) THEN
       X(3) = 15.0
      END IF
      IF ( X(3) .GE. 1440.0) THEN
       X(3) = 1440.0
      END IF
       
      IF ( X(4) .LE. 0.001) THEN
       X(4) = 0.001
      END IF       

C     BEGIN THE CALCULATION BASED ON CONVOLUTION EQUATION
      DO 2010, I = 1, 96
      T = I * 15.0
   
      Y1 = X(1) -  T
      Y2 = X(1) +  T
      Y3 = 2 * SQRT(X(2) * T)
      Y4 = EERFC(Y1/Y3)
      Y5 = EERFC(Y2/Y3)
      Y6 = EEXP((X(1)) / X(2))
      
      TERM1 = 0.5 * X(4) * (Y4 - Y6 * Y5)

      IF ((T - X(3)) .LE. 0.00) THEN
       TERM2 = 0.00
       GOTO 300
      ELSE 
       Z1 = X(1) - (T - X(3))
       Z2 = X(1) + (T - X(3))
       Z3 = 2*SQRT(X(2)*(T - X(3)))
       Z4 = EERFC(Z1/Z3)
       Z5 = EERFC(Z2/Z3)
       Z6 = EEXP((X(1)) / X(2))
       TERM2 = 0.5 * X(4) * (Z4 - Z6 * Z5)
      END IF

300   IF ( (TERM1-TERM2) .LE. 1E-15 ) THEN
       RAIN_MODEL(I) = 1E-15
      ELSE      
       RAIN_MODEL(I) = TERM1 - TERM2 
      END IF 
    
      VOLUME_MODEL = VOLUME_MODEL + RAIN_MODEL(I)
      VOLUME_REALDATA = VOLUME_REALDATA + RAIN_REALDATA(I)
      
2010  CONTINUE 
C     IF MODEL DATA IS NOT GENERATED, STOP THE PROGRAM.     
      IF (VOLUME_MODEL .EQ. 0.00) THEN
       WRITE (*,*) 'VOLUME_MODEL IS 0.00. PRESS ANY KEY TO STOP.'
       PAUSE
       STOP
      END IF
      V_DIFFERENCE = ABS(VOLUME_MODEL - VOLUME_REALDATA)
     +       / VOLUME_MODEL * 100

C     TARGET FUNCTION
      TARGET = V_DIFFERENCE        
       
      RETURN     

      END

C     CONDITIONAL ERROR FUCNTION
      REAL FUNCTION EERFC(Q1)
      REAL*8 Q1

      IF (Q1 .GE. 4.0) THEN 
       EERFC = 0.00
       GOTO 400
      END IF
      IF (Q1 .LE. -4.0) THEN
       EERFC = 2.00
       GOTO 400
      END IF
      IF (Q1 .LT. 0.00) THEN
       EERFC = 2.00 - ERFC(-1.0*Q1)
       GOTO 400
      END IF
      IF ((Q1 .GE. 0.00) .AND. (Q1 .LT. 4.00)) THEN
       EERFC = ERFC(Q1)
       GOTO 400
      END IF
      
400   RETURN
      END       

C     APPROXIMATE ERROR FUNCTION
      REAL FUNCTION ERFC(Q2)
      REAL A1, A2, A3, A4, A5, A6
      REAL *8 Q2, TEMP1, TEMP2

      A1 = 0.0705230784
      A2 = 0.0422820183
      A3 = 0.0092705272
      A4 = 0.0001520143
      A5 = 0.0002765672
      A6 = 0.0000430638
      
      TEMP1=1+(A1*Q2)+(A2*(Q2**2))+(A3*(Q2**3))
     +      +(A4*(Q2**4))+(A5*(Q2**5))+(A6*(Q2**6))
      IF (TEMP1 .EQ. 0.00) THEN
       TEMP1 = 1.0E-2
      END IF 
      TEMP2= TEMP1**16
      ERFC = 1 / TEMP2

      RETURN
      END
      
C     CONDITIONAL EXPONENTIAL FUNCTION
      FUNCTION EEXP(Q3)
      REAL Q3
      
      IF (Q3 .LT. -100.0) THEN
       EEXP = 0.0000
       GOTO 500
      END IF
      IF (Q3 .GT. 85.0) THEN
       EEXP = EXP(85.0)
       GOTO 500
      END IF
      EEXP = EXP(Q3)
500   RETURN
      END       
                             